#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const yargs_1 = require("yargs");
const yargs_support_1 = require("./util/yargs-support");
const index_1 = require("./index");
function withBaseOptions(yargs) {
    yargs
        .option(yargs_support_1.buildOptionWithEnvDefault('APIKEY', {
        alias: 'apiKey',
        describe: 'ApiKey for IBM Cloud login. Can also be provided as an environment property',
        required: true,
        type: 'string',
    }))
        .option(yargs_support_1.buildOptionWithEnvDefault('RESOURCE_GROUP', {
        alias: ['resourceGroup', 'rg'],
        describe: 'The IBM Cloud resource group for the login. Can also be provided as an environment property',
        required: true,
        type: 'string',
    }))
        .option(yargs_support_1.buildOptionWithEnvDefault('REGION', {
        alias: ['region', 'r'],
        describe: 'The IBM Cloud region for the login. The value defaults to "us-south" if not provided',
        type: 'string',
        default: 'us-south',
    }))
        .option(yargs_support_1.buildOptionWithEnvDefault('REGISTRY_URL', {
        alias: 'registry',
        describe: 'The host name for the IBM Cloud image registry. The value defaults to "us.icr.io" if not provided',
        type: 'string',
        default: 'us.icr.io',
    }))
        .option(yargs_support_1.buildOptionWithEnvDefault('REGISTRY_NAMESPACE', {
        alias: 'namespace',
        describe: 'The namespace to use within the IBM Cloud image registry. The value defaults to "default" if not provided',
        type: 'string',
        default: 'default',
    }))
        .option(yargs_support_1.buildOptionWithEnvDefault('IMAGE_NAME', {
        alias: ['imageName', 'image'],
        required: true,
        describe: 'The name of the image that will be built. Can be provided as an environment variable',
        type: 'string',
    }))
        .option(yargs_support_1.buildOptionWithEnvDefault('IMAGE_VERSION', {
        alias: ['imageVersion', 'ver'],
        required: true,
        describe: 'The version of the image that will be built. Can be provided as an environment variable',
        type: 'string',
    }))
        .option(yargs_support_1.buildOptionWithEnvDefault('IMAGE_BUILD_NUMBER', {
        alias: 'buildNumber',
        describe: 'The buildNumber that will be added to the image version, if provided. Can be provided as an environment variable',
        type: 'string',
    }))
        .option('debug', {
        describe: 'Turn on extra logging',
        type: 'boolean',
    })
        .option('quiet', {
        describe: 'Suppress all logging',
        type: 'boolean',
    });
    return yargs;
}
yargs_1.scriptName('ibmcloud-image')
    .usage('$0 <cmd> [args]')
    .command('build [args]', 'build the image and push it into the IBM Cloud registry', (argv) => withBaseOptions(argv), (argv) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    if (argv.debug) {
        console.log('arguments', argv);
    }
    try {
        const { stdout, stderr } = yield index_1.buildImage(argv);
        if (!argv.quiet) {
            console.log(stdout);
            console.log(stderr);
        }
    }
    catch (error) {
        console.log('error', error);
        process.exit(1);
    }
}))
    .command('deploy [args]', 'deploy an image from the IBM Cloud registry into a kubernetes cluster', (argv) => withBaseOptions(argv)
    .option(yargs_support_1.buildOptionWithEnvDefault('CLUSTER_NAME', {
    alias: 'cluster',
    required: true,
    describe: 'The cluster into which the image will be deployed. Can also be provided as an environment property',
    type: 'string',
}))
    .option(yargs_support_1.buildOptionWithEnvDefault('CHART_ROOT', {
    alias: 'chartRoot',
    required: true,
    describe: 'The root directory where the chart is located, e.g. {CHART_ROOT}/{CHART_NAME}. Can also be provided as an environment property',
    type: 'string',
}))
    .option(yargs_support_1.buildOptionWithEnvDefault('CHART_NAME', {
    alias: 'chartName',
    describe: 'The name of the helm chart. Defaults to the imageName if not provided',
    type: 'string',
}))
    .option(yargs_support_1.buildOptionWithEnvDefault('ENVIRONMENT_NAME', {
    alias: ['environmentName', 'env'],
    required: true,
    describe: 'The name of the environment into which the image will be deployed. Can also be provided as an environment property',
    type: 'string',
})), (argv) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    if (argv.debug) {
        console.log('arguments', argv);
    }
    try {
        const { stdout, stderr } = yield index_1.deployImage(argv);
        if (!argv.quiet) {
            console.log(stdout);
            console.log(stderr);
        }
    }
    catch (error) {
        console.log('error', error);
        process.exit(1);
    }
}))
    .command('cr', 'run the container-registry plugin', (argv) => argv.option('debug', {
    alias: 'v',
    describe: 'Verbose logging',
    type: 'boolean'
}), (argv) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    if (argv.debug) {
        console.log('arguments', argv);
    }
    try {
        const { stdout, stderr } = yield index_1.containerRegistry(argv);
        if (!argv.quiet) {
            console.log(stdout);
            console.log(stderr);
        }
    }
    catch (error) {
        console.log('error', error);
        process.exit(1);
    }
}))
    .demandCommand()
    .help()
    .argv;
//# sourceMappingURL=script.js.map